<?php

namespace EventPortal\Domain\ActionList;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection;

class ActionList
{
    public string $status;

    public string $type;

    public Collection $tasks;

    private Collection $context;

    private bool $hasBeenResolved = false;

    public function __construct(string $status)
    {
        $this->status = $status;
        $this->context = collect();
        $this->tasks = collect();
    }

    public static function state(string $status): self
    {
        return new static($status);
    }

    public function type(string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function addTask(callable $callable): self
    {
        $this->tasks->add($callable);

        return $this;
    }

    public function context(Model ...$models): self
    {
        $this->context = collect($models);

        return $this;
    }

    public function get(): self
    {
        if ($this->hasBeenResolved === false) {
            $this->resolve();
        }

        return $this;
    }

    private function resolve(): self
    {
        $this->tasks = collect($this->tasks)
            ->transform(function ($task) {
                return app()->call($task, $this->getContext());
            })
            ->flatten(1);

        $this->hasBeenResolved = true;

        return $this;
    }

    private function getContext(): array
    {
        return collect($this->context)
            ->mapWithKeys(function (Model $model): array {
                return [$model::KEY => $model];
            })
            ->all();
    }
}
