<?php

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\ButtonDropdown;
use EventPortal\Domain\Component\Column\TextColumn;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Enums\InertiaMethod;

Table::make('event.sign-in.index')
    ->setColumns(static function (Event $event) {
        return [
            TextColumn::make('First name')->default()->searchable()->sortable(),
            TextColumn::make('Last name')->default()->searchable()->sortable(),
            TextColumn::make('Signed In')
                ->calculated()
                ->value(static function (EventDelegate $eventDelegate) use ($event): string {
                    return $eventDelegate->isSignedInAt($event) ? 'Yes' : 'No';
                })
                ->default(),
        ];
    })
    ->setRowAction(static function (EventDelegate $eventDelegate, Event $event) {
        return ButtonDropdown::make('Manage')
            ->when(
                has_permission('event.sign-delegates-in-out'),
                static function (ButtonDropdown $button) use ($event, $eventDelegate): void {
                    $button->actions(
                        [
                            Button::make($eventDelegate->isSignedInAt($event) ? 'Sign out' : 'Sign In')
                                ->method(InertiaMethod::POST())
                                ->route('admin.event.sign-in.update',
                                    ['event' => $event, 'eventDelegate' => $eventDelegate]),
                        ]
                    );
                });
    });
