<?php

namespace EventPortal\Tests\Unit\Delegate\Handlers;

use EventPortal\Domain\Address\Models\Address;
use EventPortal\Domain\Delegate\Handlers\UpdateAddress;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class UpdateAddressTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function address_is_updated()
    {
        $delegate = factory(Delegate::class)->create();

        $address = factory(Address::class)->create([
            'line_1' => '123 Fake Street',
            'line_2' => 'Knotreel Borough',
            'line_3' => 'VA',
            'city' => 'Milan',
            'area' => 'The Shire',
            'country' => 'Plumage',
            'area_code' => 'DY10 4EU',
            'addressable_type' => Delegate::class,
            'addressable_id' => $delegate->id,
        ]);

        $payload = Payload::make([
            'line_1' => '125 Fake Street',
            'country' => 'Kidderminster',
        ])->withContext(['delegate' => $delegate]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateAddress])
            ->thenReturn();

        $this->assertDatabaseHas('addresses', [
            'line_1' => '125 Fake Street',
            'line_2' => 'Knotreel Borough',
            'line_3' => 'VA',
            'city' => 'Milan',
            'area' => 'The Shire',
            'country' => 'Kidderminster',
            'area_code' => 'DY10 4EU',
        ]);

        $this->assertInstanceOf(Delegate::class, $payload->getContext('delegate'));
    }

    /** @test **/
    public function handles_nested_address()
    {
        $delegate = factory(Delegate::class)->create();

        $address = factory(Address::class)->create([
            'line_1' => '123 Fake Street',
            'line_2' => 'Knotreel Borough',
            'line_3' => 'VA',
            'city' => 'Milan',
            'area' => 'The Shire',
            'country' => 'Plumage',
            'area_code' => 'DY10 4EU',
            'addressable_type' => Delegate::class,
            'addressable_id' => $delegate->id,
        ]);

        $payload = Payload::make([
            'address' => [
                'line_1' => '125 Fake Street',
                'country' => 'Kidderminster',
            ],
        ])->withContext(['delegate' => $delegate]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateAddress])
            ->thenReturn();

        $this->assertDatabaseHas('addresses', [
            'line_1' => '125 Fake Street',
            'line_2' => 'Knotreel Borough',
            'line_3' => 'VA',
            'city' => 'Milan',
            'area' => 'The Shire',
            'country' => 'Kidderminster',
            'area_code' => 'DY10 4EU',
        ]);

        $this->assertInstanceOf(Delegate::class, $payload->getContext('delegate'));
    }
}
